import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:latlong2/latlong.dart';
import 'package:http/http.dart' as http;

class BuildingsService {
  static Future<List<Building>> fetchBuildings() async {
    var buildings = (await _fetchBuildings()).toList();
    await _fetchBuildingHours(buildings);
    return buildings;
  }

  static Future<Iterable<Building>> _fetchBuildings() async {
    final response = await http
        .get(Uri.parse("https://cs.furman.edu/~csdaemon/FUNow/buildingGet.php"));

    if (response.statusCode == 200) {
      // If the server did return a 200 OK response,
      // then parse the JSON.
      final buildingsJson = jsonDecode(response.body);
      return (buildingsJson["results"] as List<dynamic>).map((json) =>
          Building.fromJson(json)
      );
    } else {
      // If the server did not return a 200 OK response,
      // then throw an exception.
      throw Exception('Failed to load buildings.');
    }
  }

  static Future<void> _fetchBuildingHours(List<Building> buildings) async {
    final response = await http
        .get(Uri.parse("https://cs.furman.edu/~csdaemon/FUNow/hoursGet.php"));

    if (response.statusCode == 200) {
      // If the server did return a 200 OK response,
      // then parse the JSON.
      final hoursListJson = jsonDecode(response.body);
      for (var hoursJson in (hoursListJson["results"] as List<dynamic>)) {
        try {
          var hours = BuildingHours.fromJson(hoursJson);
          buildings.firstWhere((building) => building.id == hours.id).hoursList.add(hours);
        } on ArgumentError {
          // if json parse fails, just move on
          // this is terribly hacky but the server is a mess I cannot fix
        }
        for (var building in buildings) { building.hoursList.sort(); }
      }
    } else {
      // If the server did not return a 200 OK response,
      // then throw an exception.
      throw Exception('Failed to load building hours.');
    }
  }
}

class Building {
  int id;
  String name;
  String? shortName;
  String category;
  String? location;
  LatLng mapLocation;
  int frequency;
  List<BuildingHours> hoursList = <BuildingHours>[];

  Building({
    required this.id,
    required this.name,
    this.shortName,
    required this.category,
    this.location,
    required this.mapLocation,
    required this.frequency,
  });

  // bool get isOpen {
  //   for(var hours in hoursList) {
  //     var now = DateTime.now();
  //     var today = Weekday.values[now.weekday - 1];
  //     if (hours.daysOfWeek.days[today] == true) {
  //       var currentTime = Duration(hours: now.hour, minutes: now.minute, seconds: now.second);
  //       if(hours.startTime != null && hours.endTime != null) {
  //         if (hours.startTime! < currentTime && hours.endTime! > currentTime) {
  //           return true;
  //         }
  //       }
  //     }
  //   }
  //   return false;
  // }

  factory Building.fromJson(Map<String, dynamic> json) {
    return Building(
      id: int.parse(json["buildingID"]),
      name: json["name"],
      shortName: json["nickname"],
      category: json["category"],
      location: json["location"],
      mapLocation: LatLng(double.parse(json["latitude"]), double.parse(json["longitude"])),
      frequency: int.parse(json["frequency"]),
    );
  }
}

class BuildingHours implements Comparable<BuildingHours> {
  final int id;
  final String? meal;
  final TimeOfDay startTime;
  final TimeOfDay endTime;
  final String daysOfWeek;
  final int dayOrder;

  BuildingHours({
    required this.id,
    this.meal,
    required this.startTime,
    required this.endTime,
    required this.daysOfWeek,
    required this.dayOrder,
  });

  static TimeOfDay _getDurationFromString(String s) {
    var timeComponents = s.split(":");
    var hour = int.parse(timeComponents[0]);
    var minute = int.parse(timeComponents[1]);

    return TimeOfDay(hour: hour, minute: minute);
  }

  factory BuildingHours.fromJson(Map<String, dynamic> json) {
    if (json["Start"] == null || json["End"] == null) {
      throw ArgumentError("Start and end time cannot be null");
    }

    return BuildingHours(
      id: int.parse(json["buildingID"]),
      meal: json["meal"],
      startTime: _getDurationFromString(json["Start"]),
      endTime: _getDurationFromString(json["End"]),
      daysOfWeek: json["day"],
      dayOrder: int.parse(json["dayorder"]),
    );
  }

  @override
  int compareTo(BuildingHours other) {
    return dayOrder - other.dayOrder;
  }
}
