import 'dart:convert';

import 'package:external_app_launcher/external_app_launcher.dart';
import 'package:flutter/material.dart';
import 'package:flutter_remix/flutter_remix.dart';
import 'package:http/http.dart' as http;
import 'package:url_launcher/url_launcher.dart';

class HealthSafetyService {
  static Future<Iterable<HealthSafetyInfo>> fetchHealthSafetyInfo() async {
    final response = await http
      .get(Uri.parse("https://cs.furman.edu/~csdaemon/FUNow/healthSafetyGet.php"));

    if (response.statusCode == 200) {
      // If the server did return a 200 OK response,
      // then parse the JSON.
      final healthSafetyJson = jsonDecode(response.body);
      return (healthSafetyJson["results"] as List<dynamic>).map((json) =>
        HealthSafetyInfo.fromJson(json)
      );
    } else {
      // If the server did not return a 200 OK response,
      // then throw an exception.
      throw Exception('Failed to load health and safety info.');
    }
  }
}

class HealthSafetyInfo {
  int id;
  String name;
  HealthSafetyInfoType type;
  String content;
  IconData icon;

  HealthSafetyInfo({
    required this.id,
    required this.name,
    required this.type,
    required this.content,
    required this.icon,
  });

  String get linkText {
    switch(type) {
      case HealthSafetyInfoType.link:
        return "External Link";
      case HealthSafetyInfoType.app:
        return "Open App";
      case HealthSafetyInfoType.phone:
        return content.replaceAllMapped(
          RegExp(r'(\d{3})(\d{3})(\d+)'),
          (Match m) => "(${m[1]}) ${m[2]}-${m[3]}"
        );
      case HealthSafetyInfoType.removed:
        return "Removed";
    }
  }

  void launch() async {
    if(
      type == HealthSafetyInfoType.link ||
      type == HealthSafetyInfoType.phone
    ) {
      var url = (type == HealthSafetyInfoType.link)
        ? Uri.parse(content)
        : Uri(scheme: "tel", path: content);
      if (await canLaunchUrl(url)) {
        await launchUrl(
          url,
          mode: LaunchMode.externalApplication,
        );
      } else {
        throw "Could not launch $url";
      }
    } else if (type == HealthSafetyInfoType.app) {
      await LaunchApp.openApp(
        androidPackageName: content.split("https://")[1],
        iosUrlScheme: 'pulsesecure://',
        appStoreLink: 'itms-apps://itunes.apple.com/us/app/pulse-secure/id945832041',
      );
    }
  }
  
  factory HealthSafetyInfo.fromJson(Map<String, dynamic> json) {
    getTypeFromString(String typeName) {
      switch(typeName) {
        case "phone":
          return HealthSafetyInfoType.phone;
        case "link":
          return HealthSafetyInfoType.link;
        case "app":
          return HealthSafetyInfoType.app;
        case "removed":
          return HealthSafetyInfoType.removed;
        default:
          throw "Invalid health safety info type value \"$typeName\"";
      }
    }

    Map<String, IconData> iconMap = {
      "shield": FlutterRemix.shield_star_line,
      "staroflife": FlutterRemix.star_line,
      "car": FlutterRemix.car_line,
      "bandage.fill": Icons.healing,
      "person.circle": FlutterRemix.user_line,
      "heart.circle": FlutterRemix.heart_line,
    };

    return HealthSafetyInfo(
      id: int.parse(json["id"]),
      name: json["name"],
      type: getTypeFromString(json["type"]),
      content: json["content"],
      icon: iconMap[json["icon"]] ?? FlutterRemix.heart_line,
    );
  }
}

enum HealthSafetyInfoType {
  app,
  phone,
  link,
  removed,
}

